// SPDX-License-Identifier: MIT
/**
 * Reset Edgelock se05x.
 * Copyright (c) 2022 Atmark Techno,Inc.
 */
#include <errno.h>
#include <getopt.h>
#include <limits.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <se05x/nxLog_sss.h> // for calling nLog() from ex_sss_main_inc.h
#include <se05x/ex_sss_boot.h>
#include <se05x/nxEnsure.h>
#include <se05x/se05x_APDU.h>

/* used by ex_sss_main_inc.h */
static ex_sss_boot_ctx_t gex_sss_boot_ctx;
#define EX_SSS_BOOT_PCONTEXT (&gex_sss_boot_ctx)
#define EX_SSS_BOOT_DO_ERASE 0
#define EX_SSS_BOOT_EXPOSE_ARGC_ARGV 1

#include <ex_sss_main_inc.h>

#if !defined(VERSION_STRING)
#define VERSION_STRING "for testing"
#endif

static struct option options[] = { { "help", no_argument, 0, 'h' },
				   { "yes", no_argument, 0, 'y' },
				   { "version", no_argument, 0, 'V' },
				   { NULL, no_argument, NULL, 0 } };

static void usage(FILE *stream)
{
	fputs("\
Usage: se05x_reset [options] port\n",
	      stream);

	fputs("\
Delete selectively secure objects and crypto objects from se05x by using \n\
Se05x_API_DeleteAll_Iterative().\n",
	      stream);

	fputs("\
  port             device file and address for T=1 over I2C.\n",
	      stream);

	fputs("\
  options:\n\
  -y --yes	   execute command.\n\
  -V --version	   display version number.\n\
  -h --help	   display help message.\n",
	      stream);
}

static void print_version(void)
{
	printf("se05x_reset %s\n", VERSION_STRING);
	printf("copyright (c) 2022 Atmark Techno,Inc.\n");
}

sss_status_t ex_sss_entry(ex_sss_boot_ctx_t *pCtx)
{
	int argc = gex_sss_argc;
	char *const *argv = (void *)gex_sss_argv;
	int c;
	bool execute = false;
	smStatus_t status = SM_NOT_OK;
	sss_se05x_session_t *pSession = (sss_se05x_session_t *)&pCtx->session;

	while ((c = getopt_long(argc, argv, "hVy", options, NULL)) != -1) {
		switch (c) {
		case 0:
			break;
		case 'h':
			usage(stdout);
			return kStatus_SSS_Success;
		case 'V':
			print_version();
			return kStatus_SSS_Success;
		case 'y':
			execute = true;
			break;
		default:
			fprintf(stderr, "invalid option.\n");
			usage(stderr);
			return kStatus_SSS_Fail;
		}
	}

	if ((argc - optind) > 1) {
		fprintf(stderr, "incorrect number of arguments.\n");
		return kStatus_SSS_Fail;
	}

	if (execute) {
		status = Se05x_API_DeleteAll_Iterative(&pSession->s_ctx);
		ENSURE_OR_GO_EXIT(status == SM_OK);
		printf("secure objects have been successfully removed.\n");
		return kStatus_SSS_Success;
	}
	fprintf(stderr, "To reset se05x, requires -y option. \n\n");
	usage(stderr);
	return kStatus_SSS_Fail;
exit:
	fprintf(stdout, "failure occurred in sss api (%x)\n", status);
	return kStatus_SSS_Fail;
}

